<?php
class RSAUtil {
    /**
     * 读取私钥 
     * 注：opessl 3.0及以上版本使用openssl_pkcs12_read读取pfx证书会有异常，建议使用pem格式证书
     * @param string $pfxPath 私钥文件路径
     * @param string $privateKeyPass 私钥密码
     * @return string 私钥
     * @throws Exception
     */
    private static function getPrivateKey($pfxPath, $privateKeyPass) {
        if (!file_exists($pfxPath) || !is_readable($pfxPath)) {
            throw new Exception("私钥文件不存在或不可读！路径：" . $pfxPath);
        }

        $pkcs12 = file_get_contents($pfxPath);
        $privateKey = [];
        if (openssl_pkcs12_read($pkcs12, $privateKey, $privateKeyPass)) {
            return $privateKey['pkey'];
        } else {
            throw new Exception("私钥证书读取出错！原因[证书或密码不匹配]，请检查本地证书相关信息。");
        }
    }

    /**
     * 读取公钥
     * @param string $publicPath 公钥文件路径
     * @return resource 公钥
     * @throws Exception
     */
    private static function getPublicKey($publicPath) {
        if (!file_exists($publicPath) || !is_readable($publicPath)) {
            throw new Exception("公钥文件不存在或不可读！路径：" . $publicPath);
        }

        $keyFile = file_get_contents($publicPath);
        $publicKey = openssl_get_publickey($keyFile);
        if (!$publicKey) {
            throw new Exception("公钥不可用！路径：" . $publicPath);
        }

        return $publicKey;
    }

    /**
     * 公钥加密
     * @param string $data 加密数据
     * @param string $publicPath 公钥文件路径
     * @return string 加密后的数据
     * @throws Exception
     */
    public static function encryptByPublicKeyFile($data, $publicPath) {
        $key = self::getPublicKey($publicPath);
        return self::encrypt($data, $key, false);
    }

    /**
     * 私钥加密
     * @param string $data 加密数据
     * @param string $pfxPath 私钥文件路径
     * @param string $privateKeyPass 私钥密码
     * @return string 加密后的数据
     * @throws Exception
     */
    public static function encryptByPrivateKeyFile($data, $pfxPath, $privateKeyPass) {
        $key = self::getPrivateKey($pfxPath, $privateKeyPass);
        return self::encrypt($data, $key, true);
    }

    /**
     * 加密数据
     * @param string $data 加密数据
     * @param resource|string $key 公钥或私钥
     * @param bool $isPrivateKey 是否为私钥
     * @return string 加密后的数据
     * @throws Exception
     */
    private static function encrypt($data, $key, $isPrivateKey) {
        $base64Data = base64_encode($data);
        $encryptedData = '';
        $blockSize = self::getKeySize($key, $isPrivateKey) / 8 - 11;

        for ($offset = 0; $offset < strlen($base64Data); $offset += $blockSize) {
            $block = substr($base64Data, $offset, $blockSize);
            $encryptedBlock = '';
            if ($isPrivateKey) {
                openssl_private_encrypt($block, $encryptedBlock, $key);
            } else {
                openssl_public_encrypt($block, $encryptedBlock, $key);
            }
            $encryptedData .= bin2hex($encryptedBlock);
        }

        return $encryptedData;
    }

    /**
     * 私钥解密
     * @param string $data 解密数据
     * @param string $pfxPath 私钥文件路径
     * @param string $privateKeyPass 私钥密码
     * @return string 解密后的数据
     * @throws Exception
     */
    public static function decryptByPrivateKeyFile($data, $pfxPath, $privateKeyPass) {
        $key = self::getPrivateKey($pfxPath, $privateKeyPass);
        return self::decrypt($data, $key, true);
    }

    /**
     * 公钥解密
     * @param string $data 解密数据
     * @param string $publicPath 公钥文件路径
     * @return string 解密后的数据
     * @throws Exception
     */
    public static function decryptByPublicKeyFile($data, $publicPath) {
        $key = self::getPublicKey($publicPath);
        return self::decrypt($data, $key, false);
    }

    /**
     * 解密数据
     * @param string $data 解密数据
     * @param resource|string $key 公钥或私钥
     * @param bool $isPrivateKey 是否为私钥
     * @return string 解密后的数据
     * @throws Exception
     */
    private static function decrypt($data, $key, $isPrivateKey) {
        $decryptedData = '';
        $blockSize = self::getKeySize($key, $isPrivateKey) / 4;

        for ($offset = 0; $offset < strlen($data); $offset += $blockSize) {
            $block = hex2bin(substr($data, $offset, $blockSize));
            $decryptedBlock = '';
            if ($isPrivateKey) {
                openssl_private_decrypt($block, $decryptedBlock, $key);
            } else {
                openssl_public_decrypt($block, $decryptedBlock, $key);
            }
            $decryptedData .= $decryptedBlock;
        }

        return base64_decode($decryptedData);
    }

    /**
     * 获取密钥长度
     * @param resource|string $key 密钥
     * @param bool $isPrivateKey 是否为私钥
     * @return int 密钥长度
     * @throws Exception
     */
    private static function getKeySize($key, $isPrivateKey) {
        $keyDetails = $isPrivateKey
            ? openssl_pkey_get_details(openssl_pkey_get_private($key))
            : openssl_pkey_get_details(openssl_pkey_get_public($key));

        if ($keyDetails && array_key_exists('bits', $keyDetails)) {
            return $keyDetails['bits'];
        }

        throw new Exception("无效的密钥");
    }
	
	/**
     * 私钥签名
     * @param string $data 待签名数据
     * @param string $pfxPath 私钥文件路径
     * @param string $privateKeyPass 私钥密码
     * @param int $signType 签名类型 OPENSSL_ALGO_SHA1|OPENSSL_ALGO_SHA256
     * @return string 签名
     * @throws Exception
     */
    public static function rsaSign($data, $pfxPath, $privateKeyPass, $signType) {
        $privateKey = self::getPrivateKey($pfxPath, $privateKeyPass);
        if (!openssl_sign($data, $signature, $privateKey, $signType)) {
            throw new Exception("签名失败！");
        }
        return bin2hex($signature);
    }

    /**
     * 公钥验签
     * @param string $data 原文数据
     * @param string $signature 签名
     * @param string $publicPath 公钥文件路径
     * @param int $signType 验签类型 OPENSSL_ALGO_SHA1|OPENSSL_ALGO_SHA256
     * @return bool 验签是否通过
     * @throws Exception
     */
    public static function rsaVerifySignature($data, $signature, $publicPath, $signType) {
        $publicKey = self::getPublicKey($publicPath);
        $result = openssl_verify($data, hex2bin($signature), $publicKey, $signType);
        if ($result == -1) {
            throw new Exception("验签发生错误！");
        }
        return $result == 1 ? true : false;
    }

}
